/*
* Copyright © 2010 Red Hat, Inc.
*
* Licensed under the Apache License, Version 2.0 (the "License");
* you may not use this file except in compliance with the License.
* You may obtain a copy of the License at
*
*           http://www.apache.org/licenses/LICENSE-2.0
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an "AS IS" BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
*/

package com.redhat.rhevm.api.powershell.resource;

import java.util.ArrayList;
import java.util.List;

import javax.ws.rs.core.UriInfo;

import com.redhat.rhevm.api.common.util.JAXBHelper;
import com.redhat.rhevm.api.common.util.LinkHelper;
import com.redhat.rhevm.api.model.ActionsBuilder;
import com.redhat.rhevm.api.model.Link;
import com.redhat.rhevm.api.model.StorageDomain;
import com.redhat.rhevm.api.model.StorageDomainType;
import com.redhat.rhevm.api.model.Template;
import com.redhat.rhevm.api.model.Templates;
import com.redhat.rhevm.api.model.VM;
import com.redhat.rhevm.api.model.VMs;
import com.redhat.rhevm.api.resource.AssignedPermissionsResource;
import com.redhat.rhevm.api.resource.StorageDomainContentsResource;
import com.redhat.rhevm.api.resource.FilesResource;
import com.redhat.rhevm.api.resource.StorageDomainResource;
import com.redhat.rhevm.api.powershell.model.PowerShellStorageDomain;
import com.redhat.rhevm.api.powershell.util.PowerShellCmd;
import com.redhat.rhevm.api.powershell.util.PowerShellParser;
import com.redhat.rhevm.api.powershell.util.PowerShellPool;
import com.redhat.rhevm.api.powershell.util.PowerShellPoolMap;
import com.redhat.rhevm.api.powershell.util.PowerShellUtils;

public class PowerShellStorageDomainResource extends AbstractPowerShellActionableResource<StorageDomain> implements StorageDomainResource {

    private PowerShellStorageDomainsResource parent;

    public PowerShellStorageDomainResource(String id,
                                           PowerShellStorageDomainsResource parent,
                                           PowerShellPoolMap shellPools,
                                           PowerShellParser parser) {
        super(id, parent.getExecutor(), parent, shellPools, parser);
        this.parent = parent;
    }

    public static List<StorageDomain> runAndParse(PowerShellPool pool, PowerShellParser parser, String command) {
        List<PowerShellStorageDomain> storageDomains =
            PowerShellStorageDomain.parse(parser, PowerShellCmd.runCommand(pool, command));
        List<StorageDomain> ret = new ArrayList<StorageDomain>();
        for (PowerShellStorageDomain storageDomain : storageDomains) {
            ret.add(storageDomain);
        }
        return ret;
    }

    /**
     * Run a powershell command and parse the output as a single storage
     * domain.
     *
     * @param command the powershell command to execute
     * @param whether the 'sharedStatus' property is needed
     * @return a single storage domain, or null
     */
    public static StorageDomain runAndParseSingle(PowerShellPool pool, PowerShellParser parser, String command) {
        List<StorageDomain> storageDomains = runAndParse(pool, parser, command);

        return !storageDomains.isEmpty() ? storageDomains.get(0) : null;
    }

    public StorageDomain runAndParseSingle(String command) {
        return runAndParseSingle(getPool(), getParser(), command);
    }

    public static StorageDomain addLinks(UriInfo uriInfo, StorageDomain storageDomain) {
        storageDomain = JAXBHelper.clone("storage_domain", StorageDomain.class, storageDomain);

        if (isExport(storageDomain)) {
            String [] subCollections = { "templates", "vms" };

            storageDomain.getLinks().clear();

            for (String collection : subCollections) {
                addSubCollection(uriInfo, storageDomain, collection);
            }
        }

        storageDomain = LinkHelper.addLinks(uriInfo, storageDomain);

        String [] subCollections = { "files" };

        for (String collection : subCollections) {
            Link link = new Link();
            link.setRel(collection);
            link.setHref(LinkHelper.getUriBuilder(uriInfo, storageDomain).path(collection).build().toString());
            storageDomain.getLinks().add(link);
        }

        ActionsBuilder actionsBuilder = new ActionsBuilder(LinkHelper.getUriBuilder(uriInfo, storageDomain),
                                                           StorageDomainResource.class);
        storageDomain.setActions(actionsBuilder.build());

        return storageDomain;
    }

    private static boolean isExport(StorageDomain storageDomain) {
        return StorageDomainType.fromValue(storageDomain.getType()) == StorageDomainType.EXPORT;
    }

    private static void addSubCollection(UriInfo uriInfo, StorageDomain storageDomain, String collection) {
        Link link = new Link();
        link.setRel(collection);
        link.setHref(LinkHelper.getUriBuilder(uriInfo, storageDomain).path(collection).build().toString());
        storageDomain.getLinks().add(link);
    }

    @Override
    public StorageDomain get() {
        return addLinks(getUriInfo(), runAndParseSingle("get-storagedomain " + PowerShellUtils.escape(getId())));
    }

    @Override
    public StorageDomain update(StorageDomain storageDomain) {
        validateUpdate(storageDomain);

        StringBuilder buf = new StringBuilder();

        buf.append("$d = get-storagedomain " + PowerShellUtils.escape(getId()) + ";");

        if (storageDomain.isSetName()) {
            buf.append("$d.name = " + PowerShellUtils.escape(storageDomain.getName()) + ";");
        }

        buf.append("update-storagedomain -storagedomainobject $d");

        return addLinks(getUriInfo(), runAndParseSingle(buf.toString()));
    }

    @Override
    public AssignedPermissionsResource getPermissionsResource() {
        return null;
    }

    public StorageDomainContentsResource<VMs, VM> getStorageDomainVmsResource() {
        PowerShellStorageDomainVmsResource resource =
            new PowerShellStorageDomainVmsResource(this, shellPools, getParser());
        resource.setUriInfo(getUriInfo());
        return resource;
    }

    public StorageDomainContentsResource<Templates, Template> getStorageDomainTemplatesResource() {
        PowerShellStorageDomainTemplatesResource resource =
            new PowerShellStorageDomainTemplatesResource(this, shellPools, getParser());
        resource.setUriInfo(getUriInfo());
        return resource;
    }

    @Override
    public FilesResource getFilesResource() {
        return new PowerShellFilesResource(getId(), getShellPools(), getParser(), getUriProvider());
    }
}
