/*
* Copyright © 2010 Red Hat, Inc.
*
* Licensed under the Apache License, Version 2.0 (the "License");
* you may not use this file except in compliance with the License.
* You may obtain a copy of the License at
*
*           http://www.apache.org/licenses/LICENSE-2.0
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an "AS IS" BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
*/

package com.redhat.rhevm.api.powershell.util;

import java.util.LinkedList;
import java.util.List;
import java.util.concurrent.AbstractExecutorService;
import java.util.concurrent.RejectedExecutionException;
import java.util.concurrent.TimeUnit;

/**
 * A simple executor for testing purposes that allows finegrained control
 * of when a task is run.
 */
public class ControllableExecutor extends AbstractExecutorService {

    private List<Runnable> tasks;
    private boolean shutdown = false;

    public ControllableExecutor() {
        tasks = new LinkedList<Runnable>();
    }

    @Override
    public synchronized void execute(Runnable command) {
        if (shutdown) {
            throw new RejectedExecutionException("executor is shutdown");
        }
        tasks.add(tasks.size(), command);
    }

    public void runNext() {
        Runnable task = null;
        synchronized (this) {
            if (!tasks.isEmpty()) {
                task = tasks.remove(0);
            }
        }
        if (task != null) {
            task.run();
        }
    }

    public synchronized int taskCount() {
        return tasks.size();
    }

    private void awaitTermination() {
        while (taskCount() > 0) {
            runNext();
        }
    }

    @Override
    public boolean awaitTermination(long timeout, TimeUnit unit) {
        awaitTermination();
        return true;
    }

    @Override
    public boolean isTerminated() {
        return isShutdown() && taskCount() == 0;
    }

    @Override
    public synchronized boolean isShutdown() {
        return shutdown;
    }

    @Override
    public synchronized void shutdown() {
        shutdown = true;
    }

    @Override
    public List<Runnable> shutdownNow() {
        shutdown();
        awaitTermination();
        return null;
    }
}
